==> timbits-0.3.0.tgz: package/lib/timbits.js <==
(function() {
  var Log, assets, ck, config, connectESI, express, fs, jsonp, log, pantry, path, querystring, request, server, url, views;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  fs = require('fs');
  path = require('path');
  querystring = require('querystring');
  url = require('url');
  views = require('./timbits-views');
  ck = require('coffeekup');
  Log = require('coloured-log');
  assets = require('connect-assets');
  connectESI = require('connect-esi');
  express = require('express');
  pantry = require('pantry');
  request = require('request');
  jsonp = require('jsonp-filter');
  config = {
    appName: "Timbits",
    engine: "coffee",
    port: 5678,
    home: process.cwd(),
    maxAge: 60
  };
  server = {};
  log = new Log();
  this.serve = function(options) {
    var helper_path, helpers, key, timbit_path, value;
    for (key in options) {
      value = options[key];
      config[key] = value;
    }
    this.server = express.createServer();
    log = new Log(this.server.settings.env === 'development' ? Log.DEBUG : Log.INFO);
    this.server.register('.coffee', ck);
    this.server.set('views', "" + config.home + "/views");
    this.server.set('view engine', config.engine);
    this.server.set('view options', {
      layout: false
    });
    this.server.set('jsonp callback', true);
    this.server.configure(__bind(function() {
      this.server.use(jsonp.setupJSONP());
      this.server.use(connectESI.setupESI());
      this.server.use(express.static("" + config.home + "/public"));
      this.server.use(express.static(path.join(path.dirname(__filename), "../resources")));
      this.server.use(assets({
        src: "" + config.home + "/public"
      }));
      this.server.use(assets({
        src: "" + config.home + "/views"
      }));
      this.server.use(express.bodyParser());
      return this.server.use(express.cookieParser());
    }, this));
    this.server.configure('development', __bind(function() {
      return this.server.use(express.errorHandler({
        dumpExceptions: true,
        showStack: true
      }));
    }, this));
    this.server.configure('production', __bind(function() {
      return this.server.use(express.errorHandler());
    }, this));
    this.server.all('/', function(req, res) {
      return res.redirect('/timbits/help');
    });
    this.server.all('/timbits/help', __bind(function(req, res) {
      return res.send(ck.render(views.help, {
        box: this.box
      }));
    }, this));
    this.server.all('/timbits/test', __bind(function(req, res) {
      var master, pending, timbit, _results;
      res.setHeader('Content-Type', 'text/html; charset=UTF-8');
      master = '';
      pending = Object.keys(this.box).length;
      if (pending) {
        _results = [];
        for (timbit in this.box) {
          _results.push(request({
            uri: "http://" + req.headers.host + "/" + timbit + "/test"
          }, function(error, response, body) {
            master += body;
            if (!--pending) {
              return res.end(master);
            }
          }));
        }
        return _results;
      } else {
        master += ck.render(views.test, {});
        return res.end(master);
      }
    }, this));
    helper_path = "" + config.home + "/helpers";
    helpers = {};
    fs.readdir(helper_path, __bind(function(err, files) {
      var file, helper_name, _i, _len, _results;
      if (err) {
        throw err;
      }
      _results = [];
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        _results.push(file.match(/\.(coffee|js)$/) != null ? (helper_name = file.substring(0, file.lastIndexOf(".")), log.info("Loading dynamic helpers: " + helper_name), helpers[helper_name] = require("" + helper_path + "/" + file), this.server.helpers(helpers)) : void 0);
      }
      return _results;
    }, this));
    timbit_path = "" + config.home + "/timbits";
    fs.readdir(timbit_path, __bind(function(err, files) {
      var file, _i, _len;
      if (err) {
        throw err;
      }
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        if (file.match(/\.(coffee|js)$/) != null) {
          this.add(file.substring(0, file.lastIndexOf(".")), require("" + timbit_path + "/" + file));
        }
      }
      return this.server.post('*', function(req, res) {
        var attrname;
        delete req.body.submit;
        for (attrname in req.body) {
          req.query[attrname] = req.body[attrname];
        }
        if (req.url.indexOf('?') !== -1) {
          req.url = req.url.substring(0, req.url.indexOf('?'));
        }
        return res.redirect(("" + req.url) + (req.query != null ? "?" + (querystring.stringify(req.query)) : ""));
      });
    }, this));
    try {
      this.server.listen(process.env.PORT || process.env.C9_PORT || config.port);
      log.info("Timbits server listening on port " + (this.server.address().port) + " in " + this.server.settings.env + " mode");
      server.address = this.server.address().address;
      server.port = this.server.address().port;
    } catch (err) {
      log.error("Server could not start on port " + (process.env.PORT || process.env.C9_PORT || config.port) + ". (" + err + ")");
      console.log("\nPress Ctrl+C to Exit");
      process.kill(process.pid, 'SIGTERM');
    }
    return this.server;
  };
  this.box = {};
  this.add = function(name, timbit) {
    var _ref, _ref2;
    log.notice("Placing " + name + " in the box");
    timbit.name = name;
    if ((_ref = timbit.viewBase) == null) {
      timbit.viewBase = name;
    }
    if ((_ref2 = timbit.maxAge) == null) {
      timbit.maxAge = config.maxAge;
    }
    this.box[name] = timbit;
    this.server.all("/" + name + "/help", function(req, res) {
      var renderHelp;
      renderHelp = function() {
        return res.send(ck.render(views.timbit_help, timbit));
      };
      return timbit.listviews(function(views) {
        timbit.views = views;
        return renderHelp();
      });
    });
    this.server.all("/" + name + "/test", function(req, res) {
      return timbit.test("http://" + req.headers.host, timbit, function(results) {
        return res.send(ck.render(views.timbit_test, results));
      });
    });
    return this.server.all("/" + name + "/:view?", function(req, res) {
      var attr, context, k, key, v, value, _base, _ref3, _ref4, _ref5, _ref6, _ref7;
      try {
        context = {};
        _ref3 = req.query;
        for (k in _ref3) {
          v = _ref3[k];
          context[k.toLowerCase()] = v;
        }
        context.name = timbit.name;
        context.view = "" + timbit.viewBase + "/" + ((_ref4 = (_base = req.params).view) != null ? _ref4 : _base.view = 'default');
        context.maxAge = timbit.maxAge;
        _ref5 = timbit.params;
        for (k in _ref5) {
          attr = _ref5[k];
          key = k.toLowerCase();
          if ((_ref6 = context[key]) == null) {
            context[key] = attr["default"];
          }
          value = context[key];
          if (value) {
            if ((_ref7 = attr.type) == null) {
              attr.type = 'String';
            }
            switch (attr.type.toLowerCase()) {
              case 'number':
                context[key] = Number(value);
                if (isNaN(context[key])) {
                  throw "" + value + " is not a valid Number for " + key;
                }
                break;
              case 'boolean':
                if (value.toLowerCase() === 'true') {
                  context[key] = true;
                } else if (value.toLowerCase() === 'false') {
                  context[key] = false;
                } else {
                  throw "" + value + " is not a valid value for " + key + ".  Must be true or false.";
                }
                break;
              case 'date':
                context[key] = Date.parse(value);
                if (isNaN(context[key])) {
                  throw "" + value + " is not a valid Date for " + key;
                }
            }
          }
          if (attr.required && !value) {
            throw "" + key + " is a required parameter";
          }
          if (value && attr.strict && attr.values.indexOf(value) === -1) {
            throw "" + value + " is not a valid value for " + key + ".  Must be one of [" + (attr.values.join()) + "]";
          }
          if (value instanceof Array && !attr.multiple) {
            throw "" + key + " must be a single value";
          }
        }
        return timbit.eat(req, res, context);
      } catch (ex) {
        log.error("" + req.url + " - " + ex);
        throw ex;
      }
    });
  };
  this.Timbit = (function() {
    function Timbit() {}
    Timbit.prototype.log = log;
    Timbit.prototype.render = function(req, res, context) {
      var output;
      res.setHeader("Cache-Control", "max-age=" + context.maxAge);
      res.setHeader("Edge-Control", "max-age=" + context.maxAge);
      if (context.remote === 'true') {
        output = "context_" + context.esi_id + " = " + (JSON.stringify(context)) + ";\nvar views = views || {};\nif (views['" + context.view + "'] || false) {\n	render = CoffeeKup.render(views['" + context.view + "'], context_" + context.esi_id + ");\n	" + (context.esi_id != null ? "$('#" + context.esi_id + "').html(render)" : "$('body').append(render)") + ";\n}\nelse {\n	$.ajax({\n		url: \"http://" + req.headers.host + "/" + context.view + ".coffee?json=true&callback=?\",\n		dataType: \"jsonp\",\n		cache: true,\n		jsonpCallback: \"" + (context.view.replace(/\//, '_')) + "\",\n		success: function(data, textStatus, jqXHR){\n			views['" + context.view + "'] = data; //save view for reuse\n			render = CoffeeKup.render(views['" + context.view + "'], context_" + context.esi_id + ");\n			" + (context.esi_id != null ? "$('#" + context.esi_id + "').html(render)" : "$('body').append(render)") + ";\n		}\n	});\n}";
        res.setHeader("Content-Type", "text/javascript");
        res.write(output);
        return res.end();
      } else if (/^\w+\/json$/.test(context.view)) {
        return res.json(context);
      } else {
        return res.render(context.view, context);
      }
    };
    Timbit.prototype.fetch = function(req, res, context, options, callback) {
      var name;
      if (callback == null) {
        callback = this.render;
      }
      name = options.name || 'data';
      return pantry.fetch(options, __bind(function(error, results) {
        context[name] = results;
        return callback(req, res, context);
      }, this));
    };
    Timbit.prototype.eat = function(req, res, context) {
      return this.render(req, res, context);
    };
    Timbit.prototype.listviews = function(callback) {
      var view;
      view = [];
      return fs.readdir("" + config.home + "/views/" + this.viewBase, function(err, list) {
        var file, _fn, _i, _len;
        if (err || list === void 0) {
          view.push('default');
        } else {
          _fn = function(file) {
            if (file.match(/\.coffee/) != null) {
              return view.push(file.substring(0, file.lastIndexOf(".")));
            }
          };
          for (_i = 0, _len = list.length; _i < _len; _i++) {
            file = list[_i];
            _fn(file);
          }
        }
        return callback(view);
      });
    };
    Timbit.prototype.test = function(host, context, callback) {
      var displayTests, results, runTests, testParams, testQueries, testRequest, testRunExamples, testRunQueries;
      results = {
        timbit: context.name,
        views: [],
        required: [],
        optional: [],
        queries: [],
        tests: [],
        warnings: []
      };
      testParams = function() {
        var k, param, v, _ref, _results;
        _ref = context.params;
        _results = [];
        for (k in _ref) {
          v = _ref[k];
          param = "" + k + "=" + v.values[0];
          _results.push(v.required ? results.required.push(param) : results.optional.push(param));
        }
        return _results;
      };
      testQueries = function() {
        return results.queries.push("" + (results.required.concat(results.optional).join('&')));
      };
      testRequest = function(type, uri, callback) {
        return request({
          uri: "" + uri
        }, function(error, response, body) {
          results.tests.push({
            type: type,
            uri: uri,
            status: response.statusCode,
            error: (error != null ? error : '')
          });
          return callback();
        });
      };
      testRunQueries = function(callback) {
        var pending, query, view, _i, _j, _len, _len2, _ref, _ref2, _ref3, _results, _results2;
        if (((_ref = results.queries) != null ? _ref.length : void 0) === 0) {
          pending = results.views.length;
          _ref2 = results.views;
          _results = [];
          for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
            view = _ref2[_i];
            _results.push((function(view) {
              return testRequest("views", "" + host + "/" + context.name + "/" + view, function() {
                if (!--pending) {
                  return callback();
                }
              });
            })(view));
          }
          return _results;
        } else {
          pending = results.queries.length * results.views.length;
          _ref3 = results.queries;
          _results2 = [];
          for (_j = 0, _len2 = _ref3.length; _j < _len2; _j++) {
            query = _ref3[_j];
            _results2.push((function(query) {
              var view, _k, _len3, _ref4, _results3;
              _ref4 = results.views;
              _results3 = [];
              for (_k = 0, _len3 = _ref4.length; _k < _len3; _k++) {
                view = _ref4[_k];
                _results3.push((function(view) {
                  return testRequest("queries", "" + host + "/" + context.name + "/" + view + "?" + query, function() {
                    if (!--pending) {
                      return callback();
                    }
                  });
                })(view));
              }
              return _results3;
            })(query));
          }
          return _results2;
        }
      };
      testRunExamples = function(callback) {
        var example, pending, _i, _len, _ref, _results;
        if (context.examples != null) {
          pending = context.examples.length;
          _ref = context.examples;
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            example = _ref[_i];
            _results.push((function(example) {
              return testRequest("example", "" + host + example.href, function() {
                if (!--pending) {
                  return callback();
                }
              });
            })(example));
          }
          return _results;
        } else {
          return callback();
        }
      };
      runTests = function() {
        testParams();
        testQueries();
        return testRunQueries(function() {
          return testRunExamples(function() {
            return displayTests();
          });
        });
      };
      displayTests = function() {
        var test, warning, _i, _j, _len, _len2, _ref, _ref2;
        log.notice("Testing Timbit - " + context.name);
        _ref = results.tests;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          test = _ref[_i];
          if (test.status >= 400 || (typeof error !== "undefined" && error !== null)) {
            log.error("Test: " + test.type + " URI: " + test.uri + " Status: " + test.status + " Error: " + test.error);
          } else {
            log.info("Test: " + test.type + " URI: " + test.uri + " Status: " + test.status);
          }
        }
        _ref2 = results.warnings;
        for (_j = 0, _len2 = _ref2.length; _j < _len2; _j++) {
          warning = _ref2[_j];
          log.warning("Message: " + warning.message);
        }
        return callback(results);
      };
      return this.listviews(function(views) {
        results.views = views;
        return runTests();
      });
    };
    return Timbit;
  })();
}).call(this);

==> timbits-0.3.1.tgz: package/lib/timbits.js <==
(function() {
  var Log, assets, ck, config, connectESI, express, fs, jsonp, log, pantry, path, querystring, request, server, url, views;
  var __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  fs = require('fs');
  path = require('path');
  querystring = require('querystring');
  url = require('url');
  views = require('./timbits-views');
  ck = require('coffeekup');
  Log = require('coloured-log');
  assets = require('connect-assets');
  connectESI = require('connect-esi');
  express = require('express');
  pantry = require('pantry');
  request = require('request');
  jsonp = require('jsonp-filter');
  config = {
    appName: "Timbits",
    engine: "coffee",
    port: 5678,
    home: process.cwd(),
    maxAge: 60
  };
  server = {};
  log = new Log();
  this.serve = function(options) {
    var helper_path, helpers, key, timbit_path, value;
    for (key in options) {
      value = options[key];
      config[key] = value;
    }
    this.server = express.createServer();
    log = new Log(this.server.settings.env === 'development' ? Log.DEBUG : Log.INFO);
    this.server.register('.coffee', ck);
    this.server.set('views', "" + config.home + "/views");
    this.server.set('view engine', config.engine);
    this.server.set('view options', {
      layout: false
    });
    this.server.set('jsonp callback', true);
    this.server.configure(__bind(function() {
      this.server.use(express.bodyParser());
      this.server.use(express.cookieParser());
      this.server.use(express.session({
        secret: "secret"
      }));
      this.server.use(jsonp.setupJSONP());
      this.server.use(connectESI.setupESI());
      this.server.use(express.static("" + config.home + "/public"));
      this.server.use(express.static(path.join(path.dirname(__filename), "../resources")));
      this.server.use(assets({
        src: "" + config.home + "/public"
      }));
      return this.server.use(assets({
        src: "" + config.home + "/views"
      }));
    }, this));
    this.server.configure('development', __bind(function() {
      return this.server.use(express.errorHandler({
        dumpExceptions: true,
        showStack: true
      }));
    }, this));
    this.server.configure('production', __bind(function() {
      return this.server.use(express.errorHandler());
    }, this));
    this.server.all('/', function(req, res) {
      return res.redirect('/timbits/help');
    });
    this.server.get('/timbits/json', __bind(function(req, res) {
      return res.json(this.box);
    }, this));
    this.server.all('/timbits/help', __bind(function(req, res) {
      return res.send(ck.render(views.help, {
        box: this.box
      }));
    }, this));
    this.server.all('/timbits/test', __bind(function(req, res) {
      var master, pending, timbit, _results;
      res.setHeader('Content-Type', 'text/html; charset=UTF-8');
      master = '';
      pending = Object.keys(this.box).length;
      if (pending) {
        _results = [];
        for (timbit in this.box) {
          _results.push(request({
            uri: "http://" + req.headers.host + "/" + timbit + "/test"
          }, function(error, response, body) {
            master += body;
            if (!--pending) {
              return res.end(master);
            }
          }));
        }
        return _results;
      } else {
        master += ck.render(views.test, {});
        return res.end(master);
      }
    }, this));
    helper_path = "" + config.home + "/helpers";
    helpers = {};
    fs.readdir(helper_path, __bind(function(err, files) {
      var file, helper_name, _i, _len, _results;
      if (err) {
        throw err;
      }
      _results = [];
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        _results.push(file.match(/\.(coffee|js)$/) != null ? (helper_name = file.substring(0, file.lastIndexOf(".")), log.info("Loading dynamic helpers: " + helper_name), helpers[helper_name] = require("" + helper_path + "/" + file), this.server.helpers(helpers)) : void 0);
      }
      return _results;
    }, this));
    timbit_path = "" + config.home + "/timbits";
    fs.readdir(timbit_path, __bind(function(err, files) {
      var file, _i, _len, _results;
      if (err) {
        throw err;
      }
      _results = [];
      for (_i = 0, _len = files.length; _i < _len; _i++) {
        file = files[_i];
        _results.push(file.match(/\.(coffee|js)$/) != null ? this.add(file.substring(0, file.lastIndexOf(".")), require("" + timbit_path + "/" + file)) : void 0);
      }
      return _results;
    }, this));
    try {
      this.server.listen(process.env.PORT || process.env.C9_PORT || config.port);
      log.info("Timbits server listening on port " + (this.server.address().port) + " in " + this.server.settings.env + " mode");
      server.address = this.server.address().address;
      server.port = this.server.address().port;
    } catch (err) {
      log.error("Server could not start on port " + (process.env.PORT || process.env.C9_PORT || config.port) + ". (" + err + ")");
      console.log("\nPress Ctrl+C to Exit");
      process.kill(process.pid, 'SIGTERM');
    }
    return this.server;
  };
  this.box = {};
  this.add = function(name, timbit) {
    var _ref, _ref2;
    log.notice("Placing " + name + " in the box");
    timbit.name = name;
    if ((_ref = timbit.viewBase) == null) {
      timbit.viewBase = name;
    }
    if ((_ref2 = timbit.maxAge) == null) {
      timbit.maxAge = config.maxAge;
    }
    timbit.listviews(__bind(function(views) {
      return timbit.views = views;
    }, this));
    this.box[name] = timbit;
    this.server.all("/" + name + "/help", function(req, res) {
      var renderHelp;
      renderHelp = function() {
        return res.send(ck.render(views.timbit_help, timbit));
      };
      return timbit.listviews(function(views) {
        timbit.views = views;
        return renderHelp();
      });
    });
    this.server.all("/" + name + "/test", function(req, res) {
      return timbit.test("http://" + req.headers.host, timbit, function(results) {
        return res.send(ck.render(views.timbit_test, results));
      });
    });
    return this.server.all("/" + name + "/:view?", function(req, res) {
      var attr, context, k, key, v, value, _base, _ref3, _ref4, _ref5, _ref6, _ref7;
      try {
        context = {};
        _ref3 = req.query;
        for (k in _ref3) {
          v = _ref3[k];
          context[k.toLowerCase()] = v;
        }
        context.name = timbit.name;
        context.view = "" + timbit.viewBase + "/" + ((_ref4 = (_base = req.params).view) != null ? _ref4 : _base.view = 'default');
        context.maxAge = timbit.maxAge;
        _ref5 = timbit.params;
        for (k in _ref5) {
          attr = _ref5[k];
          key = k.toLowerCase();
          if ((_ref6 = context[key]) == null) {
            context[key] = attr["default"];
          }
          value = context[key];
          if (value) {
            if ((_ref7 = attr.type) == null) {
              attr.type = 'String';
            }
            switch (attr.type.toLowerCase()) {
              case 'number':
                context[key] = Number(value);
                if (isNaN(context[key])) {
                  throw "" + value + " is not a valid Number for " + key;
                }
                break;
              case 'boolean':
                if (value.toLowerCase() === 'true') {
                  context[key] = true;
                } else if (value.toLowerCase() === 'false') {
                  context[key] = false;
                } else {
                  throw "" + value + " is not a valid value for " + key + ".  Must be true or false.";
                }
                break;
              case 'date':
                context[key] = Date.parse(value);
                if (isNaN(context[key])) {
                  throw "" + value + " is not a valid Date for " + key;
                }
            }
          }
          if (attr.required && !value) {
            throw "" + key + " is a required parameter";
          }
          if (value && attr.strict && attr.values.indexOf(value) === -1) {
            throw "" + value + " is not a valid value for " + key + ".  Must be one of [" + (attr.values.join()) + "]";
          }
          if (value instanceof Array && !attr.multiple) {
            throw "" + key + " must be a single value";
          }
        }
        return timbit.eat(req, res, context);
      } catch (ex) {
        log.error("" + req.url + " - " + ex);
        throw ex;
      }
    });
  };
  this.Timbit = (function() {
    function Timbit() {}
    Timbit.prototype.log = log;
    Timbit.prototype.render = function(req, res, context) {
      var output;
      res.setHeader("Cache-Control", "max-age=" + context.maxAge);
      res.setHeader("Edge-Control", "max-age=" + context.maxAge);
      if (context.remote === 'true') {
        output = "context_" + context.esi_id + " = " + (JSON.stringify(context)) + ";\nvar views = views || {};\nif (views['" + context.view + "'] || false) {\n	render = CoffeeKup.render(views['" + context.view + "'], context_" + context.esi_id + ");\n	" + (context.esi_id != null ? "$('#" + context.esi_id + "').html(render)" : "$('body').append(render)") + ";\n}\nelse {\n	$.ajax({\n		url: \"http://" + req.headers.host + "/" + context.view + ".coffee?json=true&callback=?\",\n		dataType: \"jsonp\",\n		cache: true,\n		jsonpCallback: \"" + (context.view.replace(/\//, '_')) + "\",\n		success: function(data, textStatus, jqXHR){\n			views['" + context.view + "'] = data; //save view for reuse\n			render = CoffeeKup.render(views['" + context.view + "'], context_" + context.esi_id + ");\n			" + (context.esi_id != null ? "$('#" + context.esi_id + "').html(render)" : "$('body').append(render)") + ";\n		}\n	});\n}";
        res.setHeader("Content-Type", "text/javascript");
        res.write(output);
        return res.end();
      } else if (/^\w+\/json$/.test(context.view)) {
        return res.json(context);
      } else {
        return res.render(context.view, context);
      }
    };
    Timbit.prototype.fetch = function(req, res, context, options, callback) {
      var name;
      if (callback == null) {
        callback = this.render;
      }
      name = options.name || 'data';
      return pantry.fetch(options, function(error, results) {
        if (context[name] != null) {
          if (Object.prototype.toString.call(context[name][0]) === "[object Array]") {
            context[name].push(results);
            context["" + name + "_uri"].push(options.uri);
          } else {
            context[name] = [context[name], results];
            context["" + name + "_uri"] = [context["" + name + "_uri"], options.uri];
          }
        } else {
          context[name] = results;
          context["" + name + "_uri"] = options.uri;
        }
        return callback(req, res, context);
      });
    };
    Timbit.prototype.eat = function(req, res, context) {
      return this.render(req, res, context);
    };
    Timbit.prototype.listviews = function(callback) {
      var view;
      view = [];
      return fs.readdir("" + config.home + "/views/" + this.viewBase, function(err, list) {
        var file, _fn, _i, _len;
        if (err || list === void 0) {
          view.push('default');
        } else {
          _fn = function(file) {
            if (file.match(/\.coffee/) != null) {
              return view.push(file.substring(0, file.lastIndexOf(".")));
            }
          };
          for (_i = 0, _len = list.length; _i < _len; _i++) {
            file = list[_i];
            _fn(file);
          }
        }
        return callback(view);
      });
    };
    Timbit.prototype.test = function(host, context, callback) {
      var displayTests, results, runTests, testParams, testQueries, testRequest, testRunExamples, testRunQueries;
      results = {
        timbit: context.name,
        views: [],
        required: [],
        optional: [],
        queries: [],
        tests: [],
        warnings: []
      };
      testParams = function() {
        var k, param, v, _ref, _results;
        _ref = context.params;
        _results = [];
        for (k in _ref) {
          v = _ref[k];
          param = "" + k + "=" + v.values[0];
          _results.push(v.required ? results.required.push(param) : results.optional.push(param));
        }
        return _results;
      };
      testQueries = function() {
        return results.queries.push("" + (results.required.concat(results.optional).join('&')));
      };
      testRequest = function(type, uri, callback) {
        return request({
          uri: "" + uri
        }, function(error, response, body) {
          results.tests.push({
            type: type,
            uri: uri,
            status: response.statusCode,
            error: (error != null ? error : '')
          });
          return callback();
        });
      };
      testRunQueries = function(callback) {
        var pending, query, view, _i, _j, _len, _len2, _ref, _ref2, _ref3, _results, _results2;
        if (((_ref = results.queries) != null ? _ref.length : void 0) === 0) {
          pending = results.views.length;
          _ref2 = results.views;
          _results = [];
          for (_i = 0, _len = _ref2.length; _i < _len; _i++) {
            view = _ref2[_i];
            _results.push((function(view) {
              return testRequest("views", "" + host + "/" + context.name + "/" + view, function() {
                if (!--pending) {
                  return callback();
                }
              });
            })(view));
          }
          return _results;
        } else {
          pending = results.queries.length * results.views.length;
          _ref3 = results.queries;
          _results2 = [];
          for (_j = 0, _len2 = _ref3.length; _j < _len2; _j++) {
            query = _ref3[_j];
            _results2.push((function(query) {
              var view, _k, _len3, _ref4, _results3;
              _ref4 = results.views;
              _results3 = [];
              for (_k = 0, _len3 = _ref4.length; _k < _len3; _k++) {
                view = _ref4[_k];
                _results3.push((function(view) {
                  return testRequest("queries", "" + host + "/" + context.name + "/" + view + "?" + query, function() {
                    if (!--pending) {
                      return callback();
                    }
                  });
                })(view));
              }
              return _results3;
            })(query));
          }
          return _results2;
        }
      };
      testRunExamples = function(callback) {
        var example, pending, _i, _len, _ref, _results;
        if (context.examples != null) {
          pending = context.examples.length;
          _ref = context.examples;
          _results = [];
          for (_i = 0, _len = _ref.length; _i < _len; _i++) {
            example = _ref[_i];
            _results.push((function(example) {
              return testRequest("example", "" + host + example.href, function() {
                if (!--pending) {
                  return callback();
                }
              });
            })(example));
          }
          return _results;
        } else {
          return callback();
        }
      };
      runTests = function() {
        testParams();
        testQueries();
        return testRunQueries(function() {
          return testRunExamples(function() {
            return displayTests();
          });
        });
      };
      displayTests = function() {
        var test, warning, _i, _j, _len, _len2, _ref, _ref2;
        log.notice("Testing Timbit - " + context.name);
        _ref = results.tests;
        for (_i = 0, _len = _ref.length; _i < _len; _i++) {
          test = _ref[_i];
          if (test.status >= 400 || (typeof error !== "undefined" && error !== null)) {
            log.error("Test: " + test.type + " URI: " + test.uri + " Status: " + test.status + " Error: " + test.error);
          } else {
            log.info("Test: " + test.type + " URI: " + test.uri + " Status: " + test.status);
          }
        }
        _ref2 = results.warnings;
        for (_j = 0, _len2 = _ref2.length; _j < _len2; _j++) {
          warning = _ref2[_j];
          log.warning("Message: " + warning.message);
        }
        return callback(results);
      };
      return this.listviews(function(views) {
        results.views = views;
        return runTests();
      });
    };
    return Timbit;
  })();
}).call(this);
